# -*- coding: utf-8 -*-

import os
import sys
from ctypes import *
from novas import novaslib

_neg_err = "%(name)s must be >= 0.0"
_option_err = "%(name)s must be in %(allowed)s"

def ephem_open(ephem_name=None):
    """
    This function opens a JPL planetary ephemeris file and sets initial
    values. This function must be called prior to calls to the other JPL
    ephemeris functions.

    *Parameters*
        ephem_name : string (optional)
            Name (including path if necessary) of direct-access JPL
            ephemeris file. This argument is optional if the full path
            to the ephemeris file is set in an environment variable
            called EPHEMERIS_FILE.  If EPHEMERIS_FILE is set but
            ``ephem_name`` is given when the function is called, the
            function will use the value of ``ephem_name`` and ignore
            EPHEMERIS_FILE.

    *Returns*
        jd_begin : float
            Beginning Julian date of the ephemeris file.
        jd_end : float
            Ending Julian date of the ephemeris file.
        de_number : integer
            DE number of the ephemeris file opened.

    *References*
        .. [R1] Standish, E.M. and Newhall, X X (1988). "The JPL Export
            Planetary Ephemeris"; JPL document dated 17 June 1988.

    """

    _ephem_open = novaslib.ephem_open
    _ephem_open.argtypes = (c_char_p, POINTER(c_double), POINTER(c_double),
                            POINTER(c_short))
    _ephem_open.restype = c_short

    jd_begin = c_double()
    jd_end = c_double()
    de_number = c_short()

    # Begin customizations by Brandon Rhodes for release on PyPI
    if ephem_name is None and 'EPHEMERIS_FILE' not in os.environ:
        try:
            import novas_de405
        except ImportError:
            pass
        else:
            directory = os.path.dirname(novas_de405.__file__)
            ephem_name = os.path.join(directory, 'DE405.bin').encode('utf-8')
    # End customizations by Brandon Rhodes for release on PyPI

    if ephem_name is None:
        try:
            ephem_name = os.environ['EPHEMERIS_FILE']
        except KeyError:
            message = """ephem_open needs the full path to a binary ephemeris
                         file. Either pass the path to ephem_open or set an
                         environment variable called EPHEMERIS_FILE to the
                         path. If you choose the later you may continue to call
                         ephem_open without any arguments."""
            raise IOError(message)

    return_value = _ephem_open(c_char_p(ephem_name), byref(jd_begin),
                               byref(jd_end), byref(de_number))

    if return_value == 1:
        raise IOError("Ephemeris file %s not found" % ephem_name)
    elif return_value in range(2, 11):
        raise IOError("Error reading from ephemeris file header")
    elif return_value == 11:
        raise IOError("Unable to set record length; ephemeris not supported")

    return jd_begin.value, jd_end.value, de_number.value

# Function added at user request by Brandon Rhodes 2012 Oct 2
def ephem_close():
    """
    This function closes a JPL planetary ephemeris file and frees the memory.

    *References*
        .. [R1] Standish, E.M. and Newhall, X X (1988). "The JPL Export
            Planetary Ephemeris"; JPL document dated 17 June 1988.

    """
    novaslib.ephem_close()
# End added function

def planet_ephemeris(tjd, target, center):
    """
    This function accesses the JPL planetary ephemeris to give the
    position and velocity of the target object with respect to the
    center object.

    *Parameters*
        tjd : tuple of floats, of length 2
            Julian date split into two parts. 'tjd' may be split any way
            (although the first element is usually the "integer" part,
            and the second element is the "fractional" part).  Julian
            date is in the TDB or "T_eph" time scale.
        target : {0, ..., 13}
            Number of 'target' point.
        center : {0, ..., 13}
            Number of 'center' (origin) point.

            The numbering convention for 'target' and'center' is:
                = 0 ... Mercury
                = 1 ... Venus
                = 2 ... Earth
                = 3 ... Mars
                = 4 ... Jupiter
                = 5 ... Saturn
                = 6 ... Uranus
                = 7 ... Neptune
                = 8 ... Pluto
                = 9 ... Moon
                = 10 ... Sun
                = 11 ... Solar system bary.
                = 12 ... Earth-Moon bary.
                = 13 ... Nutations (long int. and obliq.)
                (If nutations are desired, set 'target' = 13;
                 'center' will be ignored on that call.)

    *Returns*
        position : tuple of floats, of length 3
            Position vector array of target relative to center, measured
            in AU.
        velocity : tuple of floats, of length 3
            Velocity vector array of target relative to center, measured
            in AU/day.

    *References*
        .. [R1] Standish, E.M. and Newhall, X X (1988). "The JPL Export
            Planetary Ephemeris"; JPL document dated 17 June 1988.

    """

    if tjd[0] + tjd[1] < 0:
        raise ValueError(_neg_err % {'name': 'tjd[0]+tjd[1]'})
    if target not in range(0, 14):
        raise ValueError(_option_err % {'name': 'target',
                                        'allowed': range(0, 14)})
    if center not in range(0, 14):
        raise ValueError(_option_err % {'name': 'center',
                                        'allowed': range(0, 14)})

    _planet_ephemeris = novaslib.planet_ephemeris
    _planet_ephemeris.argtypes = (c_double*2, c_short, c_short,
                                  POINTER(c_double*3), POINTER(c_double*3))
    _planet_ephemeris.restype = c_short

    position = (c_double*3)()
    velocity = (c_double*3)()

    return_value = _planet_ephemeris((c_double*2)(*tjd), target, center,
                                     byref(position), byref(velocity))

    if return_value == 1:
        raise IOError("Error reading from ephemeris file")
    elif return_value == 2:
        raise ValueError("tjd[0]+tjd[1] does not fall within the time span of \
                         the open ephemeris file")

    return tuple([i for i in position]), tuple([j for j in velocity])

def state(jed, target):
    """
    This function reads and interpolates the JPL planetary ephemeris
    file.

    *Parameters*
        jed : tuple of floats, of length 2
            Julian date (TDB) at which interpolation is wanted. Any
            combination of jed[0]+jed[1] which falls within the time
            span on the file is a permissible epoch. See Note [N1]_
            below.
        target : {0, ..., 10}
            The requested body to get data for from the ephemeris file.
            The designation of the astronomical bodies is:
                = 0 ... Mercury
                = 1 ... Venus
                = 2 ... Earth-Moon barycenter
                = 3 ... Mars
                = 4 ... Jupiter
                = 5 ... Saturn
                = 6 ... Uranus
                = 7 ... Neptune
                = 8 ... Pluto
                = 9 ... geocentric Moon
                = 10 ... Sun

    *Returns*
        target_pos : tuple of floats, of length 3
            The barycentric position vector array of the requested
            object, in AU.
        target_vel : tuple of floats, of length 3
            The barycentric velocity vector array of the requested
            object, in AU/Day.

        Both vectors are referenced to the Earth mean equator and
        equinox of epoch.

    *Notes*
        .. [N1] For ease in programming, the user may put the entire
            epoch in jed[0] and set jed[1] = 0. For maximum
            interpolation accuracy set jed[0] = the most recent midnight
            at or before interpolation epoch, and set jed[1] =
            fractional part of a day elapsed between jed[0] and epoch.
            As an alternative, it may prove convenient to set jed[0] =
            some fixed epoch, such as start of the integration and
            jed[1] = elapsed interval between then and epoch.

    *References*
        .. [R1] Standish, E.M. and Newhall, X X (1988). "The JPL Export
            Planetary Ephemeris"; JPL document dated 17 June 1988.

    """

    if jed[0] + jed[1] < 0:
        raise ValueError(_neg_err % {'name': 'jed[0]+jed[1]'})
    if target not in range(0, 11):
        raise ValueError(_option_err % {'name': 'target',
                                        'allowed': range(0, 11)})

    _state = novaslib.state
    _state.argtypes = (POINTER(c_double*2), c_short, POINTER(c_double*3),
                       POINTER(c_double*3))
    _state.restype = c_short

    target_pos = (c_double*3)()
    target_vel = (c_double*3)()

    return_value = _state(byref((c_double*2)(*jed)), target, byref(target_pos),
                          byref(target_vel))

    if return_value == 1:
        raise IOError("Error reading from ephemeris file")
    elif return_value == 2:
        raise ValueError("jed[0]+jed[1] does not fall within the time span of \
                         the open ephemeris file")

    return tuple([i for i in target_pos]), tuple([j for j in target_vel])

def split(tt):
    """
    This function breaks up a float number into a double integer part
    and a fractional part.

    *Parameters*
        tt : float
            Input number.

    *Returns*
        fr : tuple of floats, of length 2
            Output tuple;
                fr[0] contains integer part,
                fr[1] contains fractional part.
            For negative input numbers,
                fr[0] contains the next more negative integer;
                fr[1] contains a positive fraction.

    *References*
        .. [R1] Standish, E.M. and Newhall, X X (1988). "The JPL Export
            Planetary Ephemeris"; JPL document dated 17 June 1988.

    """

    _split = novaslib.split
    _split.argtypes = (c_double, POINTER(c_double*2))
    _split.restype = None

    fr = (c_double*2)()

    _split(tt, byref(fr))

    return tuple([i for i in fr])


# -*- coding: utf-8 -*-

from ctypes import *
from novas import novaslib
from novas.compat.compat import _neg_err

def iau2000a(jd_high, jd_low):
    """
    Computes the forced nutation of the non-rigid Earth based on the
    IAU 2000A nutation model.
    
    *Parameters*
        jd_high : float
            High-order part of TT Julian date.
        jd_low : float
            Low-order part of TT Julian date.
    
    *Returns*
        (dpsi, deps) : tuple of floats
            Nutation (luni-solar + planetary) in (longitude, obliquity)
            in radians.
    
    *Notes*
        .. [N1] The IAU 2000A nutation model is MHB_2000 without the
            free core nutation and without the corrections to Lieske
            precession.
    
    *References*
        .. [R1] IERS Conventions (2003), Chapter 5.
        .. [R2] Simon et al. (1994) Astronomy and Astrophysics 282,
            663-683, esp. Sections 3.4-3.5.
    
    """

    if jd_high + jd_low < 0.0:
        raise ValueError(_neg_err % {'name': 'jd_high+jd_low'})

    _iau2000a = novaslib.iau2000a
    _iau2000a.argtypes = (c_double, c_double, POINTER(c_double), POINTER(c_double))
    _iau2000a.restype = None

    dpsi = c_double()
    deps = c_double()

    _iau2000a(jd_high, jd_low, byref(dpsi), byref(deps))

    return (dpsi.value, deps.value)

def iau2000b(jd_high, jd_low):
    """
    Computes the forced nutation of the non-rigid Earth based on the
    IAU 2000B precession/nutation model.
    
    *Parameters*
        jd_high : float
            High-order part of TT Julian date.
        jd_low : float
            Low-order part of TT Julian date.
    
    *Returns*
        (dpsi, deps) : tuple of floats
            Nutation (luni-solar + planetary) in (longitude, obliquity)
            in radians.
    
    *Notes*
        .. [N1] IAU 2000B reproduces the IAU 2000A model to a precision
            of 1 milliarcsecond in the interval 1995-2020.
    
    *References*
        .. [R1] McCarthy, D. and Luzum, B. (2003). "An Abridged Model of
            the Precession & Nutation of the Celestial Pole," Celestial
            Mechanics and Dynamical Astronomy, Volume 85, Issue 1,
            Jan. 2003, p. 37. (IAU 2000B)
        .. [R2] IERS Conventions (2003), Chapter 5.
    
    """

    if jd_high + jd_low < 0.0:
        raise ValueError(_neg_err % {'name': 'jd_high+jd_low'})

    _iau2000b = novaslib.iau2000b
    _iau2000b.argtypes = (c_double, c_double, POINTER(c_double), POINTER(c_double))
    _iau2000b.restype = None

    dpsi = c_double()
    deps = c_double()

    _iau2000b(jd_high, jd_low, byref(dpsi), byref(deps))

    return (dpsi.value, deps.value)

def nu2000k(jd_high, jd_low):
    """
    Computes the forced nutation of the non-rigid Earth: Model NU2000K.
    This model is a modified version of IAU 2000A, which has been
    truncated for speed of execution, and uses Simon et al. (1994)
    fundamental arguments throughout.NU2000K agrees with IAU 2000A at
    the 0.1 milliarcsecond level from 1700 to 2300.
    
    *Parameters*
        jd_high : float
            High-order part of TT Julian date.
        jd_low : float
            Low-order part of TT Julian date.
    
    *Returns*
        (dpsi, deps) : tuple of floats
            Nutation (luni-solar + planetary) in (longitude, obliquity)
            in radians.
    
    *Notes*
        .. [N1] NU2000K was compared to IAU 2000A over six centuries
            (1700-2300). The average error in dpsi is 20
            microarcseconds, with 98% of the errors < 60
            microarcseconds;the average error in deleps is 8
            microarcseconds, with 100% of the errors < 60
            microarcseconds.
        .. [N2] NU2000K was developed by G. Kaplan (USNO) in March 2004.
    
    *References*
        .. [R1] IERS Conventions (2003), Chapter 5.
        .. [R2] Simon et al. (1994) Astronomy and Astrophysics 282,
            663-683, esp. Sections 3.4-3.5.
    
    """

    if jd_high + jd_low < 0.0:
        raise ValueError(_neg_err % {'name': 'jd_high+jd_low'})
        
    _nu2000k = novaslib.nu2000k
    _nu2000k.argtypes = (c_double, c_double, POINTER(c_double), POINTER(c_double))
    _nu2000k.restype = None

    dpsi = c_double()
    deps = c_double()

    _nu2000k(jd_high, jd_low, byref(dpsi), byref(deps))

    return (dpsi.value, deps.value)


# -*- coding: utf-8 -*-

import codecs
import os
import platform
from distutils.core import setup, Extension
from distutils.core import Command
from distutils.command.build_clib import build_clib
from distutils.sysconfig import get_python_lib
from distutils import log
from asc2eph import *

class build_dynamic_clib(build_clib):
    def finalize_options (self):
        self.set_undefined_options('build',
                                   ('build_lib', 'build_clib'),
                                   ('build_temp', 'build_temp'),
                                   ('compiler', 'compiler'),
                                   ('debug', 'debug'),
                                   ('force', 'force'))

        self.libraries = self.distribution.libraries
        if self.libraries:
            self.check_library_list(self.libraries)

        if self.include_dirs is None:
            self.include_dirs = self.distribution.include_dirs or []
        if isinstance(self.include_dirs, str):
            self.include_dirs = self.include_dirs.split(os.pathsep)

    def build_libraries (self, libraries):
        for (lib_name, build_info) in libraries:
            sources = build_info.get('sources')
            if sources is None or not isinstance(sources, (list, tuple)):
                raise DistutilsSetupError(
                       "in 'libraries' option (library '%s'), "
                       "'sources' must be present and must be "
                       "a list of source filenames" % lib_name)
            sources = list(sources)

            log.info("building '%s' library", lib_name)

            macros = build_info.get('macros')
            include_dirs = build_info.get('include_dirs')
            objects = self.compiler.compile(sources,
                output_dir=self.build_temp,
                macros=macros,
                include_dirs=include_dirs,
                extra_postargs=build_info.get('extra_compile_args', []),
                debug=self.debug)

            package = build_info.get('package', '')
            self.compiler.link_shared_lib(
                objects, lib_name,
                output_dir=os.path.join(self.build_clib, package),
                extra_postargs=build_info.get('extra_link_args', []),
                debug=self.debug,)

    def run(self):
        log.info('running build_dynamic_clib')
        build_clib.run(self)

class build_ephemeris(Command):
    description = 'build a default DE405 binary for installation with the \
                   NOVAS Py package'
    
    user_options = [
        ('build-temp=', 't',
         'temporary build directory'),
        ('ephemeris-dir=', 'e',
         'ephemeris file directory')
    ]
    
    def initialize_options(self):
        self.build_temp = None
        self.ephemeris_dir = None

    def finalize_options(self):
        if self.build_temp is None:
            build = self.get_finalized_command('build')
            self.build_temp = os.path.join(build.build_temp, 'ephemeris')
            self.mkpath(self.build_temp)
            
        if self.ephemeris_dir is None:
            self.ephemeris_dir = calling_dir
        else:
            self.mkpath(os.path.abspath(os.path.join(calling_dir,
                                                     self.ephemeris_dir)))
        
    def create_ephemeris(self, de_number=405):
        try:
            download_ascii(self.build_temp, de_number)
            binary_file = open(os.path.join(self.ephemeris_dir, "DE%s.bin") %
                               de_number, 'wb')
            ncoeff = process_header(self.build_temp, de_number, binary_file)
            data_files = [os.path.join(self.build_temp, "ascp%d.%s" %
                                       (year, de_number)) for year in
                                       range(1600, 2220, 20)]
            process_data_files(data_files, ncoeff, binary_file)
            binary_file.close()
        finally:
            pass
    
    def run(self):
        log.info('running build_ephemeris')
        self.create_ephemeris()

c_sources = [
    'Cdist/solsys1.c',
    'Cdist/readeph0.c',
    'Cdist/eph_manager.c',
    'Cdist/nutation.c',
    'Cdist/novascon.c',
    'Cdist/novas.c'
]

calling_dir = None

def main():
    global calling_dir
    calling_dir = os.getcwd()

    cwd = os.path.dirname(__file__)
    if cwd:
        os.chdir(cwd)

    system = platform.system().lower()

    if 'darwin' in system:
        novaslib = [(
            'novas', {
                'package': 'novas',
                'sources': c_sources,
                'include_dirs': ['Cdist'],
                'extra_compile_args': ['-arch', 'i386',
                                       '-arch', 'x86_64',
                                       '-O2', '-Wall', '-fPIC']
            }
        )]
    elif 'windows' in system:
        raise OSError("Operating system not supported at this time")
    else:
        novaslib = [(
            'novas', {
                'package': 'novas',
                'sources': c_sources,
                'include_dirs': ['Cdist'],
                'extra_compile_args': ['-O2', '-Wall', '-fPIC']
            }
        )]

    options = {
        'name': 'NOVAS_Py',
        'version': '3.1.1.2',
        'description': "Python wrappers for the US Naval Observatory's \
                        NOVAS-C package.",
        'author': 'Eric G. Barron',
        'author_email': "%(firstdotlast)s@%(place)s" %
                        {'firstdotlast': 'eric.barron',
                         'place': 'usno.navy.mil'},
        'maintainer': 'Eric G. Barron',
        'maintainer_email': "%(firstdotlast)s@%(place)s" %
                            {'firstdotlast': 'eric.barron',
                             'place': 'usno.navy.mil'},
        'url': 'http://www.usno.navy.mil/USNO/astronomical-applications/software-products/novas',
        'download_url' : 'http://www.usno.navy.mil/USNO/astronomical-applications/software-products/novas',
        'platforms': ['macosx', 'linux'],
        'classifiers': [
            'Development Status :: 5 - Production/Stable',
            'Intended Audience :: Science/Research',
            'Natural Language :: English',
            'Operating System :: MacOS :: MacOS X',
            'Operating System :: POSIX :: Linux',
            'Programming Language :: Python :: 2.6',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3.2',
            'Programming Language :: Python :: 3.3',
            'Topic :: Scientific/Engineering :: Astronomy',
        ],

        'packages': ['novas', 'novas.compat'],
        'package_dir': {
            'novas': 'novas_py',
            'novas.compat': 'compat'
        },
        'libraries': novaslib,
        'cmdclass': {
            'build_clib': build_dynamic_clib,
            'build_ephemeris': build_ephemeris
        }
    }

    # Begin customizations by Brandon Rhodes for release on PyPI
    options['name'] = 'novas'
    options['description'] = ('The United States Naval Observatory'
                              ' NOVAS astronomy library')
    options['long_description'] = (codecs.open('README-PyPI', 'r', 'utf-8')
                                   .read())
    options['maintainer'] = (options['author'] +
                             '; packaged for PyPI by Brandon Rhodes')
    options['maintainer_email'] = 'brandon@rhodesmill.org'
    options['packages'].append('novas.tests')
    options['package_dir']['novas.tests'] = 'tests'
    del options['download_url']
    open('tests/__init__.py', 'w').close()
    import atexit
    atexit.register(os.remove, 'tests/__init__.py')
    # End customizations by Brandon Rhodes for release on PyPI

    setup(**options)

if __name__ == '__main__':
    main()



#!/usr/bin/env python

from __future__ import with_statement

import os
import re
import struct

def download_ascii(ascii_dir, de_number, verbose=True):
    import ftplib
    
    files = []
    
    if verbose:
        print("Retrieving list of ephemeris files")

    jpl = ftplib.FTP('ssd.jpl.nasa.gov')
    jpl.login()
    jpl.cwd('pub/eph/planets/ascii/de%s' % de_number)
    jpl.retrlines('LIST', files.append)
    
    for filename in [file.split()[-1] for file in files]:
        filelines = []
        if verbose:
            print("Downloading ASCII ephemeris file %s" % filename)
        jpl.retrlines('RETR %s' % filename, filelines.append)
        outfile = open(os.path.join(ascii_dir, filename), 'w')
        for line in filelines:
            outfile.write(line + '\n')
        outfile.close()
        
    jpl.close()

def process_header(ascii_dir, de_number, binary_file, verbose=True):
    group_nums = [0, 1010, 1030, 1040, 1041, 1050, 1070]
    
    header_file = open(os.path.join(ascii_dir, "header.%s" % de_number), 'r')
    header_lines = header_file.read()
    header_file.close()

    header = [group.strip(' ' + os.linesep) for group in re.split(r'GROUP\s*\d{4}', header_lines)]

    groups = dict(zip(group_nums, header))

    ksize, ncoeff = [int(number) for number in re.findall(r'\d+', groups[0])]

    alphanumeric_records = [line.strip() for line in groups[1010].split(os.linesep)]

    start = float(groups[1030].split()[0])
    end = float(groups[1030].split()[1])
    span = float(groups[1030].split()[2])

    ncon = int(groups[1040].split()[0])
    constants = dict(zip(groups[1040].split()[1:], [float(constant.replace('D', 'e')) for constant in groups[1041].split()[1:]]))

    au = float(constants['AU'])
    emrat = float(constants['EMRAT'])
    denum = int(constants['DENUM'])

    pointers = [int(pointer) for pointer in groups[1050].split()]

    for line in alphanumeric_records:
        binary_file.write(line.ljust(84))

    for key in constants.keys():
        binary_file.write(key.ljust(6))

    padding = 2400 - ncon * 6

    binary_file.write(struct.pack("=%ix" % padding))

    binary_file.write(struct.pack('=dddidd', start, end, span, ncon, au, emrat))

    for i in range(12):
        binary_file.write(struct.pack('=iii', pointers[i], pointers[i+13], pointers[i+13+13]))

    binary_file.write(struct.pack('=i', denum))

    binary_file.write(struct.pack('=iii', pointers[12], pointers[12+13], pointers[12+13+13]))

    binary_file.write(struct.pack('=5288x'))

    for key in constants.keys():
        binary_file.write(struct.pack('=d', constants[key]))

    padding = 8144 - ncon * 8

    binary_file.write(struct.pack('=%ix' % padding))
    
    return ncoeff
    
def process_data_files(data_files, ncoeff, binary_file, verbose=True):
    old_block = []
    new_block = []
    
    for file in data_files:
        if verbose:
            print("Processing ASCII ephemeris file %s" % os.path.split(file)[-1])
        with open(file, 'r') as data_file:
            for line in data_file:
                if len(line.split()) == 2:
                    if new_block != old_block:
                        for number in new_block[:ncoeff]:
                            binary_file.write(struct.pack('=d', number))
                        old_block = new_block
                        new_block = []
                    else:
                        new_block = []
                else:
                    for number in line.replace('D', 'e').split():
                        new_block.append(float(number))

    for number in new_block[:ncoeff]:
        binary_file.write(struct.pack('=d', number))
        
if __name__ == '__main__':
    import tempfile
    import shutil
    
    de_number = 405
    
    current_dir = os.getcwd()
    temp_dir = tempfile.mkdtemp()
    
    try:
        download_ascii(temp_dir, de_number)
        
        binary_file = open(os.path.join(current_dir, "DE%s.bin") % de_number, 'wb')
        
        ncoeff = process_header(temp_dir, de_number, binary_file)
        
        data_files = [os.path.join(temp_dir, "ascp%d.%s" % (year, de_number)) for year in range(1600, 2220, 20)]
        
        process_data_files(data_files, ncoeff, binary_file)
        
        binary_file.close()
    finally:
        shutil.rmtree(temp_dir)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import with_statement

import sys
if sys.version_info < (2, 7):
    try:
        import unittest2 as unittest
    except ImportError:
        print("Running these tests with Python 2.5 or 2.6 requires the unittest2 module")
        raise
else:
    import unittest

from novas import novaslib
from novas.compat import *
        
class TestPlace(unittest.TestCase):
    def setUp(self):
        self.jd_tt = 2455519.5
        # These two .encode() calls by Brandon Rhodes for Python 3:
        self.cel_object = make_object(2, 0, "Polaris", CatEntry("POLARIS".encode(), "HIP".encode(), 0, 2.530301028, 89.264109444, 44.22, -11.75, 7.56, -17.4))
        self.location = make_observer_on_surface(45.0, -75.0, 0.0, 10.0, 1010.0)
        self.delta_t = 67.0
        self.system = 0
        self.accuracy = 0
        
    def test_negative_julian_date(self):
        self.jd_tt = -2455519.5

        with self.assertRaises(ValueError):
            sky_pos = place(self.jd_tt, self.delta_t, self.cel_object, self.location, self.system)
            
    def test_system_value(self):
        self.system = 99
        
        with self.assertRaises(ValueError):
            sky_pos = place(self.jd_tt, self.delta_t, self.cel_object, self.location, self.system)
            
    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            sky_pos = place(self.jd_tt, self.delta_t, self.cel_object, self.location, self.system, self.accuracy)
            
class TestSiderealTime(unittest.TestCase):
    def setUp(self):
        self.jd_high = 2455519.0
        self.jd_low = 0.5
        self.delta_t = 67.0
        self.gst_type = 0
        self.method = 1
        self.accuracy = 0
    
    def test_negative_julian_date(self):
        self.jd_high = -2455519.0

        with self.assertRaises(ValueError):
            gst = sidereal_time(self.jd_high, self.jd_low, self.delta_t, self.gst_type, self.method)
            
    def test_gst_type_value(self):
        self.gst_type = 99
        
        with self.assertRaises(ValueError):
            gst = sidereal_time(self.jd_high, self.jd_low, self.delta_t, self.gst_type, self.method)
        
    def test_method_value(self):
        self.method = 99
        
        with self.assertRaises(ValueError):
            gst = sidereal_time(self.jd_high, self.jd_low, self.delta_t, self.gst_type, self.method)
        
    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            gst = sidereal_time(self.jd_high, self.jd_low, self.delta_t, self.gst_type, self.method, self.accuracy)
    
class TestTer2Cel(unittest.TestCase):
    def setUp(self):
        self.jd_ut_high = 2455519.0
        self.jd_ut_low = 0.5
        self.delta_t = 67.0
        self.xp = 0.0
        self.yp = 0.0
        self.vec1 = (0.0, 0.0, 0.0)
        self.method = 0
        self.option = 0
        self.accuracy = 0

    def test_negative_julian_date(self):
        self.jd_ut_high = -2455519.0

        with self.assertRaises(ValueError):
            vec2 = ter2cel(self.jd_ut_high, self.jd_ut_low, self.delta_t, self.xp, self.yp, self.vec1, self.method, self.option, self.accuracy)
            
    def test_vec1_tuple_length(self):
        self.vec1 = (0.0, 0.0, 0.0, 0.0)

        with self.assertRaises(ValueError):
            vec2 = ter2cel(self.jd_ut_high, self.jd_ut_low, self.delta_t, self.xp, self.yp, self.vec1, self.method, self.option, self.accuracy)

    def test_method_value(self):
        self.method = 99
        
        with self.assertRaises(ValueError):
            vec2 = ter2cel(self.jd_ut_high, self.jd_ut_low, self.delta_t, self.xp, self.yp, self.vec1, self.method, self.option, self.accuracy)

    def test_option_value(self):
        self.option = 99

        with self.assertRaises(ValueError):
            vec2 = ter2cel(self.jd_ut_high, self.jd_ut_low, self.delta_t, self.xp, self.yp, self.vec1, self.method, self.option, self.accuracy)
        
    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            vec2 = ter2cel(self.jd_ut_high, self.jd_ut_low, self.delta_t, self.xp, self.yp, self.vec1, self.method, self.option, self.accuracy)
    
class TestCel2Ter(unittest.TestCase):
    def setUp(self):
        self.jd_ut_high = 2455519.0
        self.jd_ut_low = 0.5
        self.delta_t = 67.0
        self.xp = 0.0
        self.yp = 0.0
        self.vec1 = (0.0, 0.0, 0.0)
        self.method = 0
        self.option = 0
        self.accuracy = 0

    def test_negative_julian_date(self):
        self.jd_ut_high = -2455519.0

        with self.assertRaises(ValueError):
            vec2 = cel2ter(self.jd_ut_high, self.jd_ut_low, self.delta_t, self.xp, self.yp, self.vec1, self.method, self.option, self.accuracy)
            
    def test_vec1_tuple_length(self):
        self.vec1 = (0.0, 0.0, 0.0, 0.0)

        with self.assertRaises(ValueError):
            vec2 = cel2ter(self.jd_ut_high, self.jd_ut_low, self.delta_t, self.xp, self.yp, self.vec1, self.method, self.option, self.accuracy)

    def test_method_value(self):
        self.method = 99
        
        with self.assertRaises(ValueError):
            vec2 = cel2ter(self.jd_ut_high, self.jd_ut_low, self.delta_t, self.xp, self.yp, self.vec1, self.method, self.option, self.accuracy)

    def test_option_value(self):
        self.option = 99

        with self.assertRaises(ValueError):
            vec2 = cel2ter(self.jd_ut_high, self.jd_ut_low, self.delta_t, self.xp, self.yp, self.vec1, self.method, self.option, self.accuracy)
        
    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            vec2 = cel2ter(self.jd_ut_high, self.jd_ut_low, self.delta_t, self.xp, self.yp, self.vec1, self.method, self.option, self.accuracy)
    
class TestEqu2Hor(unittest.TestCase):
    def setUp(self):
        self.jd_ut1 = 2455519.5
        self.delta_t = 67.0
        self.xp = 0.0
        self.yp = 0.0
        self.location = make_on_surface(40.0, -105.0, 1800.0, 20.0, 1020.0)
        self.ra = 0.0
        self.dec = 0.0
        self.ref_option = 0
        self.accuracy= 0

    def test_negative_julian_date(self):
        self.jd_ut1 = -2455519.0

        with self.assertRaises(ValueError):
            (zd, az), (rar, decr) = equ2hor(self.jd_ut1, self.delta_t, self.xp, self.yp, self.location, self.ra, self.dec, self.ref_option, self.accuracy)
    
    def test_ref_option_value(self):
        self.ref_option = 99

        with self.assertRaises(ValueError):
            (zd, az), (rar, decr) = equ2hor(self.jd_ut1, self.delta_t, self.xp, self.yp, self.location, self.ra, self.dec, self.ref_option, self.accuracy)

    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            (zd, az), (rar, decr) = equ2hor(self.jd_ut1, self.delta_t, self.xp, self.yp, self.location, self.ra, self.dec, self.ref_option, self.accuracy)

class TestTransformCat(unittest.TestCase):
    def setUp(self):
        pass
    
class TestTransformHip(unittest.TestCase):
    def setUp(self):
        pass
    
class TestAppStar(unittest.TestCase):
    def setUp(self):
        self.jd_tt = 2455519.5
        self.star = make_cat_entry("POLARIS", "HIP", 0, 2.530301028, 89.264109444, 44.22, -11.75, 7.56, -17.4)
        self.accuracy = 0
        
    def test_negative_julian_date(self):
        self.jd_tt = -2455519.5

        with self.assertRaises(ValueError):
            ra, dec = app_star(self.jd_tt, self.star)
            
    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            ra, dec = app_star(self.jd_tt, self.star, self.accuracy)
    
class TestTopoStar(unittest.TestCase):
    def setUp(self):
        self.jd_tt = 2455519.5
        self.delta_t = 67.0
        self.star = make_cat_entry("POLARIS", "HIP", 0, 2.530301028, 89.264109444, 44.22, -11.75, 7.56, -17.4)
        self.position = make_on_surface(45.0, -75.0, 0.0, 10.0, 1010.0)
        self.accuracy = 0
        
    def test_negative_julian_date(self):
        self.jd_tt = -2455519.5

        with self.assertRaises(ValueError):
            ra, dec = topo_star(self.jd_tt, self.delta_t, self.star, self.position)
            
    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            ra, dec = topo_star(self.jd_tt, self.delta_t, self.star, self.position, self.accuracy)
    
class TestVirtualStar(unittest.TestCase):
    def setUp(self):
        self.jd_tt = 2455519.5
        self.star = make_cat_entry("POLARIS", "HIP", 0, 2.530301028, 89.264109444, 44.22, -11.75, 7.56, -17.4)
        self.accuracy = 0
        
    def test_negative_julian_date(self):
        self.jd_tt = -2455519.5

        with self.assertRaises(ValueError):
            ra, dec = virtual_star(self.jd_tt, self.star)
            
    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            ra, dec = virtual_star(self.jd_tt, self.star, self.accuracy)
    
class TestLocalStar(unittest.TestCase):
    def setUp(self):
        self.jd_tt = 2455519.5
        self.delta_t = 67.0
        self.star = make_cat_entry("POLARIS", "HIP", 0, 2.530301028, 89.264109444, 44.22, -11.75, 7.56, -17.4)
        self.position = make_on_surface(45.0, -75.0, 0.0, 10.0, 1010.0)
        self.accuracy = 0
        
    def test_negative_julian_date(self):
        self.jd_tt = -2455519.5

        with self.assertRaises(ValueError):
            ra, dec = local_star(self.jd_tt, self.delta_t, self.star, self.position)
            
    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            ra, dec = local_star(self.jd_tt, self.delta_t, self.star, self.position, self.accuracy)
    
class TestAstroStar(unittest.TestCase):
    def setUp(self):
        self.jd_tt = 2455519.5
        self.star = make_cat_entry("POLARIS", "HIP", 0, 2.530301028, 89.264109444, 44.22, -11.75, 7.56, -17.4)
        self.accuracy = 0
        
    def test_negative_julian_date(self):
        self.jd_tt = -2455519.5

        with self.assertRaises(ValueError):
            ra, dec = astro_star(self.jd_tt, self.star)
            
    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            ra, dec = astro_star(self.jd_tt, self.star, self.accuracy)
    
class TestMeanStar(unittest.TestCase):
    def setUp(self):
        self.jd_tt = 2455519.5
        self.ra = 12.0
        self.dec = 0.0
        self.accuracy = 0
        
    def test_negative_julian_date(self):
        self.jd_tt = -2455519.5

        with self.assertRaises(ValueError):
            ira, idec = mean_star(self.jd_tt, self.ra, self.dec)
            
    def test_ra_out_of_range(self):
        self.ra = 25.0
        
        with self.assertRaises(ValueError):
            ira, idec = mean_star(self.jd_tt, self.ra, self.dec)
            
    def test_dec_out_of_range(self):
        self.dec = 100.0
        
        with self.assertRaises(ValueError):
            ira, idec = mean_star(self.jd_tt, self.ra, self.dec)
            
    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            ira, idec = mean_star(self.jd_tt, self.ra, self.dec, self.accuracy)
    
class TestAppPlanet(unittest.TestCase):
    def setUp(self):
        self.jd_tt = 2455519.5
        self.ss_body = make_object(0, 11, "Moon", CatEntry())
        self.accuracy = 0
        
    def test_negative_julian_date(self):
        self.jd_tt = -2455519.5

        with self.assertRaises(ValueError):
            ra, dec, dis = app_planet(self.jd_tt, self.ss_body)
            
    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            ra, dec, dis = app_planet(self.jd_tt, self.ss_body, self.accuracy)
    
class TestTopoPlanet(unittest.TestCase):
    def setUp(self):
        self.jd_tt = 2455519.5
        self.delta_t = 67.0
        self.ss_body = make_object(0, 11, "Moon", None)
        self.position = make_on_surface(45.0, -75.0, 0.0, 10.0, 1010.0)
        self.accuracy = 0
        
    def test_negative_julian_date(self):
        self.jd_tt = -2455519.5

        with self.assertRaises(ValueError):
            ra, dec, dis = topo_planet(self.jd_tt, self.delta_t, self.ss_body, self.position)
            
    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            ra, dec, dis = topo_planet(self.jd_tt, self.delta_t, self.ss_body, self.position, self.accuracy)
    
class TestVirtualPlanet(unittest.TestCase):
    def setUp(self):
        self.jd_tt = 2455519.5
        self.ss_body = make_object(0, 11, "Moon", None)
        self.accuracy = 0
        
    def test_negative_julian_date(self):
        self.jd_tt = -2455519.5

        with self.assertRaises(ValueError):
            ra, dec, dis = virtual_planet(self.jd_tt, self.ss_body)
            
    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            ra, dec, dis = virtual_planet(self.jd_tt, self.ss_body, self.accuracy)
    
class TestLocalPlanet(unittest.TestCase):
    def setUp(self):
        self.jd_tt = 2455519.5
        self.delta_t = 67.0
        self.ss_body = make_object(0, 11, "Moon", None)
        self.position = make_on_surface(45.0, -75.0, 0.0, 10.0, 1010.0)
        self.accuracy = 0
        
    def test_negative_julian_date(self):
        self.jd_tt = -2455519.5

        with self.assertRaises(ValueError):
            ra, dec, dis = local_planet(self.jd_tt, self.delta_t, self.ss_body, self.position)
            
    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            ra, dec, dis = local_planet(self.jd_tt, self.delta_t, self.ss_body, self.position, self.accuracy)
    
class TestAstroPlanet(unittest.TestCase):
    def setUp(self):
        self.jd_tt = 2455519.5
        self.ss_body = make_object(0, 11, "Moon", None)
        self.accuracy = 0
        
    def test_negative_julian_date(self):
        self.jd_tt = -2455519.5

        with self.assertRaises(ValueError):
            ra, dec, dis = astro_planet(self.jd_tt, self.ss_body)
            
    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            ra, dec, dis = astro_planet(self.jd_tt, self.ss_body, self.accuracy)
        
class TestAberration(unittest.TestCase):
    def setUp(self):
        self.position = (0.0, 0.0, 0.0)
        self.velocity = (0.0, 0.0, 0.0)
        self.lighttime = 1.0
        
    def test_position_tuple_length(self):
        self.position = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            x, y, z = aberration(self.position, self.velocity, self.lighttime)
            
    def test_velocity_tuple_length(self):
        self.velocity = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            x, y, z = aberration(self.position, self.velocity, self.lighttime)
            
    def test_negative_lighttime(self):
        self.lighttime = -1.0
        
        with self.assertRaises(ValueError):
            x, y, z = aberration(self.position, self.velocity, self.lighttime)
            
class TestBary2Obs(unittest.TestCase):
    def setUp(self):
        self.position1 = (0.0, 0.0, 0.0)
        self.position_obs = (0.0, 0.0, 0.0)
        
    def test_position1_tuple_length(self):
        self.position1 = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            position, lighttime = bary2obs(self.position1, self.position_obs)
            
    def test_position_obs_tuple_length(self):
        self.position_obs = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            position, lighttime = bary2obs(self.position1, self.position_obs)

class TestCIOBasis(unittest.TestCase):
    def setUp(self):
        self.date = 2455519.5
        self.ra_cio = 12.0
        self.system = 1
        self.accuracy = 0
        
    def test_negative_julian_date(self):
        self.date = -2455519.5
        
        with self.assertRaises(ValueError):
            x, y, z = cio_basis(self.date, self.ra_cio, self.system)
            
    def test_ra_cio_range_high(self):
        self.ra_cio = 25.0
        
        with self.assertRaises(ValueError):
            x, y, z = cio_basis(self.date, self.ra_cio, self.system)
            
    def test_ra_cio_range_low(self):
        self.ra_cio = -1.0
        
        with self.assertRaises(ValueError):
            x, y, z = cio_basis(self.date, self.ra_cio, self.system)
            
    def test_system_value(self):
        self.system = 99

        with self.assertRaises(ValueError):
            x, y, z = cio_basis(self.date, self.ra_cio, self.system)
        
    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            x, y, z = cio_basis(self.date, self.ra_cio, self.system, self.accuracy)
            
class TestDLight(unittest.TestCase):
    def setUp(self):
        self.position = (0.0, 0.0, 0.0)
        self.position_obs = (0.0, 0.0, 0.0)
        
    def test_position_tuple_length(self):
        self.position = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            diflt = d_light(self.position, self.position_obs)
            
    def test_position_obs_tuple_length(self):
        self.position_obs = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            diflt = d_light(self.position, self.position_obs)
            
class TestEcl2EquVec(unittest.TestCase):
    def setUp(self):
        self.date = 2455519.5
        self.position = (0.0, 0.0, 0.0)
        self.system = 0
        self.accuracy = 0
        
    def test_negative_julian_date(self):
        self.date = -2455519.5

        with self.assertRaises(ValueError):
            x, y, z = ecl2equ_vec(self.date, self.position)
            
    def test_position_tuple_length(self):
        self.position = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            x, y, z = ecl2equ_vec(self.date, self.position)
    
    def test_system_value(self):
        self.system = 99
        
        with self.assertRaises(ValueError):
            x, y, z = ecl2equ_vec(self.date, self.position, coord_sys=self.system)
    
    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            x, y, z = ecl2equ_vec(self.date, self.position, accuracy=self.accuracy)
            
class TestEqu2Ecl(unittest.TestCase):
    def setUp(self):
        self.date = 2455519.5
        self.ra = 12.0
        self.dec = 0.0
        self.system = 0
        self.accuracy = 0
        
    def test_negative_julian_date(self):
        self.date = -2455519.5

        with self.assertRaises(ValueError):
            elon, elat = equ2ecl(self.date, self.ra, self.dec)
            
    def test_ra_range_high(self):
        self.ra = 25.0
        
        with self.assertRaises(ValueError):
            elon, elat = equ2ecl(self.date, self.ra, self.dec)
            
    def test_ra_range_low(self):
        self.ra = -1.0
        
        with self.assertRaises(ValueError):
            elon, elat = equ2ecl(self.date, self.ra, self.dec)
    
    def test_dec_range_high(self):
        self.dec = 100.0
        
        with self.assertRaises(ValueError):
            elon, elat = equ2ecl(self.date, self.ra, self.dec)
            
    def test_dec_range_low(self):
        self.dec = -100.0
        
        with self.assertRaises(ValueError):
            elon, elat = equ2ecl(self.date, self.ra, self.dec)
            
    def test_system_value(self):
        self.system = 99
        
        with self.assertRaises(ValueError):
            elon, elat = equ2ecl(self.date, self.ra, self.dec, coord_sys=self.system)
            
    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            elon, elat = equ2ecl(self.date, self.ra, self.dec, accuracy=self.accuracy)
            
class TestEqu2EclVec(unittest.TestCase):
    def setUp(self):
        self.date = 2455519.5
        self.position = (0.0, 0.0, 0.0)
        self.system = 0
        self.accuracy = 0
        
    def test_negative_julian_date(self):
        self.date = -2455519.5

        with self.assertRaises(ValueError):
            x, y, z = equ2ecl_vec(self.date, self.position)
            
    def test_position_tuple_length(self):
        self.position = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            x, y, z = equ2ecl_vec(self.date, self.position)
            
    def test_system_value(self):
        self.system = 99
        
        with self.assertRaises(ValueError):
            x, y, z = equ2ecl_vec(self.date, self.position, coord_sys=self.system)
            
    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            x, y, z = equ2ecl_vec(self.date, self.position, accuracy=self.accuracy)
            
class TestEqu2Gal(unittest.TestCase):
    def setUp(self):
        self.ra = 12.0
        self.dec = 0.0
        
    def test_ra_range_high(self):
        self.ra = 25.0
        
        with self.assertRaises(ValueError):
            glon, glat = equ2gal(self.ra, self.dec)
    
    def test_ra_range_low(self):
        self.ra = -1.0
        
        with self.assertRaises(ValueError):
            glon, glat = equ2gal(self.ra, self.dec)
            
    def test_dec_range_high(self):
        self.dec = 100.0
        
        with self.assertRaises(ValueError):
            glon, glat = equ2gal(self.ra, self.dec)
            
    def test_dec_range_low(self):
        self.dec = -100.0
        
        with self.assertRaises(ValueError):
            glon, glat = equ2gal(self.ra, self.dec)
            
class TestEra(unittest.TestCase):
    def setUp(self):
        self.date_high = 2455519.0
        self.date_low = 0.0
        
    def test_negative_julian_date(self):
        self.date_high = -2455519.0

        with self.assertRaises(ValueError):
            theta = era(self.date_high)
            
class TestFrameTie(unittest.TestCase):
    def setUp(self):
        self.position = (0.0, 0.0, 0.0)
        self.direction = 0
        
    def test_position_tuple_length(self):
        self.position = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            x, y, z = frame_tie(self.position)
            
    def test_direction_value(self):
        self.direction = 99
        
        with self.assertRaises(ValueError):
            x, y, z = frame_tie(self.position, self.direction)
            
class TestGcrs2Equ(unittest.TestCase):
    def setUp(self):
        self.date = 2455519.5
        self.rag = 12.0
        self.decg = 0.0
        self.system = 0
        self.accuracy = 0
        
    def test_negative_julian_date(self):
        self.date = -2455519.5

        with self.assertRaises(ValueError):
            ra, dec = gcrs2equ(self.date, self.rag, self.decg)
            
    def test_rag_range_high(self):
        self.rag = 25.0
        
        with self.assertRaises(ValueError):
            ra, dec = gcrs2equ(self.date, self.rag, self.decg)
            
    def test_rag_range_low(self):
        self.rag = -1.0
        
        with self.assertRaises(ValueError):
            ra, dec = gcrs2equ(self.date, self.rag, self.decg)
    
    def test_decg_range_high(self):
        self.decg = 100.0
        
        with self.assertRaises(ValueError):
            ra, dec = gcrs2equ(self.date, self.rag, self.decg)
            
    def test_decg_range_low(self):
        self.decg = -100.0
        
        with self.assertRaises(ValueError):
            ra, dec = gcrs2equ(self.date, self.rag, self.decg)
            
    def test_system_value(self):
        self.system = 99
        
        with self.assertRaises(ValueError):
            ra, dec = gcrs2equ(self.date, self.rag, self.decg, coord_sys=self.system)
            
    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            ra, dec = gcrs2equ(self.date, self.rag, self.decg, accuracy=self.accuracy)
            
class TestGeoPosVel(unittest.TestCase):
    def setUp(self):
        self.date = 2455519.5
        self.delta_t = 67.0
        self.observer = make_observer_on_surface(40.0, -105.0, 1800.0, 20.0, 1020.0)
        self.accuracy = 0
        
    def test_negative_julian_date(self):
        self.date = -2455519.5

        with self.assertRaises(ValueError):
            position, velocity = geo_posvel(self.date, self.delta_t, self.observer)
            
    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            position, velocity = geo_posvel(self.date, self.delta_t, self.observer, self.accuracy)
            
class TestGravDef(unittest.TestCase):
    def setUp(self):
        self.date = 2455519.5
        self.position = (0.0, 0.0, 0.0)
        self.position_obs = (0.0, 0.0, 0.0)
        self.location = 0
        self.accuracy = 0
        
    def test_negative_julian_date(self):
        self.date = -2455519.5

        with self.assertRaises(ValueError):
            x, y, z = grav_def(self.date, self.position, self.position_obs, self.location)
            
    def test_position_tuple_length(self):
        self.position = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            x, y, z = grav_def(self.date, self.position, self.position_obs, self.location)
            
    def test_position_obs_tuple_length(self):
        self.position_obs = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            x, y, z = grav_def(self.date, self.position, self.position_obs, self.location)
            
    def test_location_value(self):
        self.location = 99
        
        with self.assertRaises(ValueError):
            x, y, z = grav_def(self.date, self.position, self.position_obs, self.location)
            
    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            x, y, z = grav_def(self.date, self.position, self.position_obs, self.location, self.accuracy)
            
class TestGravVec(unittest.TestCase):
    def setUp(self):
        self.position = (0.0, 0.0, 0.0)
        self.position_obs = (0.0, 0.0, 0.0)
        self.position_body = (0.0, 0.0, 0.0)
        self.rmass = 0.5
        
    def test_position_tuple_length(self):
        self.position = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            x, y, z = grav_vec(self.position, self.position_obs, self.position_body, self.rmass)
            
    def test_position_obs_tuple_length(self):
        self.position_obs = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            x, y, z = grav_vec(self.position, self.position_obs, self.position_body, self.rmass)
            
    def test_position_body_tuple_length(self):
        self.position_body = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            x, y, z = grav_vec(self.position, self.position_obs, self.position_body, self.rmass)
    
    def test_negative_rmass(self):
        self.rmass = -0.5
        
        with self.assertRaises(ValueError):
            x, y, z = grav_vec(self.position, self.position_obs, self.position_body, self.rmass)
            
class TestLightTime(unittest.TestCase):
    def setUp(self):
        self.date = 2455519.5
        self.object = make_object(0, 11, 'Moon', None)
        self.position_obs = (0.0, 0.0, 0.0)
        self.estimate = 0.1
        self.accuracy = 0
        
    def test_negative_julian_date(self):
        self.date = -2455519.5

        with self.assertRaises(ValueError):
            position, time = light_time(self.date, self.object, self.position_obs, self.estimate)
            
    def test_position_obs_tuple_length(self):
        self.position_obs = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            position, time = light_time(self.date, self.object, self.position_obs, self.estimate)
            
    def test_negative_estimate(self):
        self.estimate = -1.0
        
        with self.assertRaises(ValueError):
            position, time = light_time(self.date, self.object, self.position_obs, self.estimate)
            
    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            position, time = light_time(self.date, self.object, self.position_obs, self.estimate, self.accuracy)
            
class TestLimbAngle(unittest.TestCase):
    def setUp(self):
        self.position_obj = (0.0, 0.0, 0.0)
        self.position_obs = (0.0, 0.0, 0.0)
        
    def test_position_obj_tuple_length(self):
        self.position_obj = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            limb, nadir = limb_angle(self.position_obj, self.position_obs)
            
    def test_position_obs_tuple_length(self):
        self.position_obs = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            limb, nadir = limb_angle(self.position_obj, self.position_obs)
            
class TestMakeCatEntry(unittest.TestCase):
    def setUp(self):
        self.star_name = 'Test'
        self.catalog = 'XXX'
        self.star_num = 9999
        self.ra = 12.0
        self.dec = 0.0
        self.pm_ra = 0.0
        self.pm_dec = 0.0
        self.parallax = 100.0
        self.rad_vel = 0.0
        
    def test_ra_range_high(self):
        self.ra = 25.0
        
        with self.assertRaises(ValueError):
            star = make_cat_entry(self.star_name, self.catalog, self.star_num, self.ra, self.dec, self.pm_ra, self.pm_dec, self.parallax, self.rad_vel)
            
    def test_ra_range_low(self):
        self.ra = -1.0
        
        with self.assertRaises(ValueError):
            star = make_cat_entry(self.star_name, self.catalog, self.star_num, self.ra, self.dec, self.pm_ra, self.pm_dec, self.parallax, self.rad_vel)
            
    def test_dec_range_high(self):
        self.dec = 100.0
        
        with self.assertRaises(ValueError):
            star = make_cat_entry(self.star_name, self.catalog, self.star_num, self.ra, self.dec, self.pm_ra, self.pm_dec, self.parallax, self.rad_vel)
            
    def test_dec_range_low(self):
        self.dec = -100.0
        
        with self.assertRaises(ValueError):
            star = make_cat_entry(self.star_name, self.catalog, self.star_num, self.ra, self.dec, self.pm_ra, self.pm_dec, self.parallax, self.rad_vel)
            
    def test_negative_parallax(self):
        self.parallax = -100.0
        
        with self.assertRaises(ValueError):
            star = make_cat_entry(self.star_name, self.catalog, self.star_num, self.ra, self.dec, self.pm_ra, self.pm_dec, self.parallax, self.rad_vel)
            
class TestMakeObject(unittest.TestCase):
    def setUp(self):
        self.type = 0
        self.number = 1
        self.name = 'Test'
        self.star_data = None
        
    def test_type_value(self):
        self.type = 99
        
        with self.assertRaises(ValueError):
            object = make_object(self.type, self.number, self.name, self.star_data)
            
class TestMakeObserver(unittest.TestCase):
    pass

class TestMakeObserverAtGeocenter(unittest.TestCase):
    pass

class TestMakeObserverInSpace(unittest.TestCase):
    def setUp(self):
        self.position = (0.0, 0.0, 0.0)
        self.velocity = (0.0, 0.0, 0.0)
        
    def test_position_tuple_length(self):
        self.position = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            observer = make_observer_in_space(self.position, self.velocity)
            
    def test_velocity_tuple_length(self):
        self.velocity = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            observer = make_observer_in_space(self.position, self.velocity)
            
class TestMakeObserverOnSurface(unittest.TestCase):
    def setUp(self):
        self.latitude = 40.0
        self.longitude = -105.0
        self.height = 1800.0
        self.temperature = 20.0
        self.pressure = 1020.0
        
    def test_latitude_range_high(self):
        self.latitude = 100.0
        
        with self.assertRaises(ValueError):
            observer = make_observer_on_surface(self.latitude, self.longitude, self.height, self.temperature, self.pressure)
            
    def test_latitude_range_low(self):
        self.latitude = -100.0
        
        with self.assertRaises(ValueError):
            observer = make_observer_on_surface(self.latitude, self.longitude, self.height, self.temperature, self.pressure)
            
    def test_longitude_range_high(self):
        self.longitude = 360.0
        
        with self.assertRaises(ValueError):
            observer = make_observer_on_surface(self.latitude, self.longitude, self.height, self.temperature, self.pressure)
            
    def test_longitude_range_low(self):
        self.longitude = -360.0
        
        with self.assertRaises(ValueError):
            observer = make_observer_on_surface(self.latitude, self.longitude, self.height, self.temperature, self.pressure)
            
class TestMakeInSpace(unittest.TestCase):
    def setUp(self):
        self.position = (0.0, 0.0, 0.0)
        self.velocity = (0.0, 0.0, 0.0)
        
    def test_position_tuple_length(self):
        self.position = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            object = make_in_space(self.position, self.velocity)
            
    def test_velocity_tuple_length(self):
        self.velocity = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            object = make_in_space(self.position, self.velocity)
            
class TestMakeOnSurface(unittest.TestCase):
    def setUp(self):
        self.latitude = 40.0
        self.longitude = -105.0
        self.height = 1800.0
        self.temperature = 20.0
        self.pressure = 1020.0
        
    def test_latitude_range_high(self):
        self.latitude = 100.0
        
        with self.assertRaises(ValueError):
            observer = make_on_surface(self.latitude, self.longitude, self.height, self.temperature, self.pressure)
            
    def test_latitude_range_low(self):
        self.latitude = -100.0
        
        with self.assertRaises(ValueError):
            observer = make_on_surface(self.latitude, self.longitude, self.height, self.temperature, self.pressure)
            
    def test_longitude_range_high(self):
        self.longitude = 360.0
        
        with self.assertRaises(ValueError):
            observer = make_on_surface(self.latitude, self.longitude, self.height, self.temperature, self.pressure)
            
    def test_longitude_range_low(self):
        self.longitude = -360.0
        
        with self.assertRaises(ValueError):
            observer = make_on_surface(self.latitude, self.longitude, self.height, self.temperature, self.pressure)

class TestNutation(unittest.TestCase):
    def setUp(self):
        self.date = 2455519.5
        self.position = (0.0, 0.0, 0.0)
        self.direction = 0
        self.accuracy = 0
        
    def test_negative_julian_date(self):
        self.date = -2455519.5

        with self.assertRaises(ValueError):
            x, y, z = nutation(self.date, self.position, self.direction)
            
    def test_position_tuple_length(self):
        self.position = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            x, y, z = nutation(self.date, self.position, self.direction)
            
    def test_direction_value(self):
        self.direction = 99

        with self.assertRaises(ValueError):
            x, y, z = nutation(self.date, self.position, self.direction)
            
    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            x, y, z = nutation(self.date, self.position, self.direction, self.accuracy)
            
class TestPrecession(unittest.TestCase):
    def setUp(self):
        self.date = 2455519.5
        self.position = (0.0, 0.0, 0.0)
        self.newdate = 2455619.5
        
    def test_negative_julian_date(self):
        self.date = -2455519.5

        with self.assertRaises(ValueError):
            x, y, z = precession(self.date, self.position, self.newdate)
            
    def test_position_tuple_length(self):
        self.position = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            x, y, z = precession(self.date, self.position, self.newdate)
            
    def test_negative_julian_newdate(self):
        self.newdate = -2455619.5

        with self.assertRaises(ValueError):
            x, y, z = precession(self.date, self.position, self.newdate)
            
class TestProperMotion(unittest.TestCase):
    def setUp(self):
        self.date = 2455519.5
        self.position = (0.0, 0.0, 0.0)
        self.velocity = (0.0, 0.0, 0.0)
        self.newdate = 2455619.5
        
    def test_negative_julian_date(self):
        self.date = -2455519.5

        with self.assertRaises(ValueError):
            x, y, z = proper_motion(self.date, self.position, self.velocity, self.newdate)
            
    def test_position_tuple_length(self):
        self.position = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            x, y, z = proper_motion(self.date, self.position, self.velocity, self.newdate)
            
    def test_velocity_tuple_length(self):
        self.velocity = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            x, y, z = proper_motion(self.date, self.position, self.velocity, self.newdate)
            
    def test_negative_julian_newdate(self):
        self.newdate = -2455619.5

        with self.assertRaises(ValueError):
            x, y, z = proper_motion(self.date, self.position, self.velocity, self.newdate)
            
class TestRadVel(unittest.TestCase):
    def setUp(self):
        self.object = make_object(0, 11, 'Moon', None)
        self.position = (0.0, 0.0, 0.0)
        self.velocity = (0.0, 0.0, 0.0)
        self.velocity_obs = (0.0, 0.0, 0.0)
        self.distance_geo = 0.1
        self.distance_sun = 1.0
        self.distance_obj = 1.1
        
    def test_position_tuple_length(self):
        self.position = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            vel = rad_vel(self.object, self.position, self.velocity, self.velocity_obs, self.distance_geo, self.distance_sun, self.distance_obj)
            
    def test_velocity_tuple_length(self):
        self.velocity = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            vel = rad_vel(self.object, self.position, self.velocity, self.velocity_obs, self.distance_geo, self.distance_sun, self.distance_obj)
            
    def test_velocity_obs_tuple_length(self):
        self.velocity_obs = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            vel = rad_vel(self.object, self.position, self.velocity, self.velocity_obs, self.distance_geo, self.distance_sun, self.distance_obj)
            
    def test_negative_distance_geo(self):
        self.distance_geo = -0.1
        
        with self.assertRaises(ValueError):
            vel = rad_vel(self.object, self.position, self.velocity, self.velocity_obs, self.distance_geo, self.distance_sun, self.distance_obj)
            
    def test_negative_distance_sun(self):
        self.distance_sun = -0.1
        
        with self.assertRaises(ValueError):
            vel = rad_vel(self.object, self.position, self.velocity, self.velocity_obs, self.distance_geo, self.distance_sun, self.distance_obj)
            
    def test_negative_distance_obj(self):
        self.distance_obj = -0.1
        
        with self.assertRaises(ValueError):
            vel = rad_vel(self.object, self.position, self.velocity, self.velocity_obs, self.distance_geo, self.distance_sun, self.distance_obj)
            
class TestRadec2Vector(unittest.TestCase):
    def setUp(self):
        self.ra = 12.0
        self.dec = 0.0
        self.distance = 1.0
        
    def test_ra_range_high(self):
        self.ra = 25.0
        
        with self.assertRaises(ValueError):
            x, y, z = radec2vector(self.ra, self.dec, self.distance)
            
    def test_ra_range_low(self):
        self.ra = -1.0
        
        with self.assertRaises(ValueError):
            x, y, z = radec2vector(self.ra, self.dec, self.distance)
            
    def test_dec_range_high(self):
        self.dec = 100.0
        
        with self.assertRaises(ValueError):
            x, y, z = radec2vector(self.ra, self.dec, self.distance)
            
    def test_dec_range_low(self):
        self.dec = -100.0
        
        with self.assertRaises(ValueError):
            x, y, z = radec2vector(self.ra, self.dec, self.distance)
            
    def test_negative_distance(self):
        self.distance = -1.0
        
        with self.assertRaises(ValueError):
            x, y, z = radec2vector(self.ra, self.dec, self.distance)
            
class TestSpin(unittest.TestCase):
    def setUp(self):
        self.angle = 180.0
        self.position = (0.0, 0.0, 0.0)
        
    def test_angle_range_high(self):
        self.angle = 450.0
        
        with self.assertRaises(ValueError):
            x, y, z = spin(self.angle, self.position)
            
    def test_angle_range_low(self):
        self.angle = -90.0
        
        with self.assertRaises(ValueError):
            x, y, z = spin(self.angle, self.position)
            
    def test_position_tuple_length(self):
        self.position = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            x, y, z = spin(self.angle, self.position)
            
class TestStarvectors(unittest.TestCase):
    pass

class TestTerra(unittest.TestCase):
    def setUp(self):
        self.location = make_on_surface(40.0, -105.0, 1800.0, 20.0, 1020.0)
        self.time = 12.0
        
    def test_time_range_high(self):
        self.time = 25.0
        
        with self.assertRaises(ValueError):
            position, velocity = terra(self.location, self.time)
            
    def test_time_range_low(self):
        self.time = -1.0
        
        with self.assertRaises(ValueError):
            position, velocity = terra(self.location, self.time)
            
class TestVector2Radec(unittest.TestCase):
    def setUp(self):
        self.position = (0.0, 0.0, 0.0)
        
    def test_position_tuple_length(self):
        self.position = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            rightascension, declination = vector2radec(self.position)
            
class TestWobble(unittest.TestCase):
    def setUp(self):
        self.date = 2455519.5
        self.xp = 0.0
        self.yp = 0.0
        self.position = (0.0, 0.0, 0.0)
        
    def test_negative_julian_date(self):
        self.date = -2455519.5

        with self.assertRaises(ValueError):
            x, y, z = wobble(self.date, self.xp, self.yp, self.position)
            
    def test_position_tuple_length(self):
        self.position = (0.0, 0.0, 0.0, 0.0)
        
        with self.assertRaises(ValueError):
            x, y, z = wobble(self.date, self.xp, self.yp, self.position)

class TestCalDate(unittest.TestCase):
    def setUp(self):
        self.day = 2455519.5

    def test_negative_julian_date(self):
        self.day = -2455519.5

        with self.assertRaises(ValueError):
            date = cal_date(self.day)

class TestCelPole(unittest.TestCase):
    def setUp(self):
        self.jd = 2455519.5
        self.type = 1
        self.dpole1 = 0.0
        self.dpole2 = 0.0

    def test_negative_julian_date(self):
        self.jd = -2455519.5

        with self.assertRaises(ValueError):
            cel_pole(self.jd, self.type, self.dpole1, self.dpole2)

    def test_type_value(self):
        self.type = 99

        with self.assertRaises(ValueError):
            cel_pole(self.jd, self.type, self.dpole1, self.dpole2)
            
class TestCIOLocation(unittest.TestCase):
    def setUp(self):
        self.day = 2455519.5
        self.accuracy = 0

    def test_negative_julian_date(self):
        self.day = -2455519.5

        with self.assertRaises(ValueError):
            ra_cio = cio_location(self.day)

    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            ra_cio = cio_location(self.day, self.accuracy)
            
class TestCIORa(unittest.TestCase):
    def setUp(self):
        self.day = 2455519.5
        self.accuracy = 0

    def test_negative_julian_date(self):
        self.day = -2455519.5

        with self.assertRaises(ValueError):
            ra_cio = cio_ra(self.day)

    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            ra_cio = cio_ra(self.day, self.accuracy)
            
class TestEECT(unittest.TestCase):
    def setUp(self):
        self.jd_high = 2455519.0
        self.jd_low = 0.5
        self.accuracy = 0

    def test_negative_julian_date(self):
        self.jd_high = -2455519.0

        with self.assertRaises(ValueError):
            comp_terms = ee_ct(self.jd_high, self.jd_low)
            
    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            comp_terms = ee_ct(self.jd_high, self.jd_low, self.accuracy)
            
class TestEphemeris(unittest.TestCase):
    def setUp(self):
        self.jd = (2455519.0, 0.5)
        self.ss_body = make_object(0, 11, "Moon", None)
        self.origin = 0
        self.accuracy = 0

    def test_negative_julian_date(self):
        self.jd = (-2455519.0, 0.5)

        with self.assertRaises(ValueError):
            position, velocity = ephemeris(self.jd, self.ss_body, self.origin, self.accuracy)
            
    def test_origin_value(self):
        self.origin = 99

        with self.assertRaises(ValueError):
            position, velocity = ephemeris(self.jd, self.ss_body, self.origin, self.accuracy)

    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            position, velocity = ephemeris(self.jd, self.ss_body, self.origin, self.accuracy)

class TestETilt(unittest.TestCase):
    def setUp(self):
        self.jd = 2455519.5
        self.accuracy = 0

    def test_negative_julian_date(self):
        self.jd = -2455519.5

        with self.assertRaises(ValueError):
            mobl, tobl, ee, dpsi, deps = e_tilt(self.jd)

    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            mobl, tobl, ee, dpsi, deps = e_tilt(self.jd, self.accuracy)

class TestFundArgs(unittest.TestCase):
    pass

class TestIRAEquinox(unittest.TestCase):
    def setUp(self):
        self.day = 2455519.5
        self.equinox = 0
        self.accuracy = 0

    def test_negative_julian_date(self):
        self.day = -2455519.5

        with self.assertRaises(ValueError):
            ira_eq = ira_equinox(self.day, self.equinox)

    def test_equinox_value(self):
        self.equinox = 99

        with self.assertRaises(ValueError):
            ira_eq = ira_equinox(self.day, self.equinox)

    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            ira_eq = ira_equinox(self.day, self.equinox, self.accuracy)

class TestJulianDate(unittest.TestCase):
    def setUp(self):
        self.year = 2010
        self.month = 11
        self.day = 19
        self.hour = 0.0
        
    def test_year_lower_bound(self):
        self.year = -9999

        with self.assertRaises(ValueError):
            jd = julian_date(self.year, self.month, self.day, self.hour)

    def test_month_range_high(self):
        self.month = 13

        with self.assertRaises(ValueError):
            jd = julian_date(self.year, self.month, self.day, self.hour)
            
    def test_month_range_low(self):
        self.month = -1

        with self.assertRaises(ValueError):
            jd = julian_date(self.year, self.month, self.day, self.hour)
            
    def test_day_range_high(self):
        self.day = 32

        with self.assertRaises(ValueError):
            jd = julian_date(self.year, self.month, self.day, self.hour)
            
    def test_day_range_low(self):
        self.day = -1

        with self.assertRaises(ValueError):
            jd = julian_date(self.year, self.month, self.day, self.hour)

    def test_hour_range_high(self):
        self.hour = 24.0

        with self.assertRaises(ValueError):
            jd = julian_date(self.year, self.month, self.day, self.hour)

    def test_hour_range_low(self):
        self.hour = -1.0

        with self.assertRaises(ValueError):
            jd = julian_date(self.year, self.month, self.day, self.hour)
            
class TestMeanObliq(unittest.TestCase):
    def setUp(self):
        self.day = 2455519.5

    def test_negative_julian_date(self):
        self.day = -2455519.5

        with self.assertRaises(ValueError):
            epsilon = mean_obliq(self.day)
            
class TestNormAng(unittest.TestCase):
    pass

class TestNutationAngles(unittest.TestCase):
    def setUp(self):
        self.t = 0.0
        self.accuracy = 0

    def test_accuracy_value(self):
        self.accuracy = 99

        with self.assertRaises(ValueError):
            dpsi, deps = nutation_angles(self.t, self.accuracy)

class TestRefract(unittest.TestCase):
    def setUp(self):
        self.location = make_on_surface(40.0, -105.0, 1800.0, 20.0, 1020.0)
        self.zenith = 89.0
        self.atmosphere = 1

    def test_atmosphere_value(self):
        self.atmosphere = 99

        with self.assertRaises(ValueError):
            refraction = refract(self.location, self.zenith, self.atmosphere)

class TestTDB2TT(unittest.TestCase):
    def setUp(self):
        self.day = 2455519.5

    def test_negative_julian_date(self):
        self.day = -2455519.5

        with self.assertRaises(ValueError):
            tt_jd, secdiff = tdb2tt(self.day)

if __name__ == '__main__':
    unittest.main()


#!/usr/bin/env python
# -*- coding: utf-8 -*-

from novas.compat import topo_star, CatEntry, OnSurface
from novas.compat.eph_manager import ephem_open

jd_begin, jd_end, de_number = ephem_open()

print("JPL Ephemeris DE%i open. jd_beg = %9.2f  jd_end = %9.2f\n" %
    (de_number, jd_begin, jd_end)
    )

deltat = 60.0

tjds = [2450203.5, 2450203.5, 2450417.5, 2450300.5]

stars = [
	CatEntry("POLARIS", "HIP", 0, 2.530301028, 89.264109444, 44.22, -11.75, 7.56, -17.4),
	CatEntry("Delta ORI", "HIP", 1, 5.533444639, -0.299091944, 1.67, 0.56, 3.56, 16.0),
	CatEntry("Theta CAR", "HIP", 2, 10.715944806, -64.394450000, -18.87, 12.06, 7.43, 24.0)
]

geo_loc = OnSurface(45.0, -75.0, 0.0, 10.0, 1010.0)

for date in tjds:
	for star in stars:
		ra, dec = topo_star(date, deltat, star, geo_loc)

		print("JD = %14.6f  Star = %s" % (date, star.starname))
		print("RA = % 12.9f  Dec = % 12.8f\n" % (ra, dec))
